#pragma once
#include <atl/object.hpp>

namespace cpptesk
{
namespace ts
{

//   DFSM:  
struct scenario_init_base
{
    virtual bool operator() (int argc, char** argv) = 0;
    virtual ~scenario_init_base() {}
};

//   DFSM:  
struct scenario_finish_base
{
    virtual void operator() () = 0;
    virtual ~scenario_finish_base() {}
};

//   DFSM:  
struct scenario_get_state_base
{
    virtual Object * operator() () = 0;
    virtual ~scenario_get_state_base() {}
};

struct default_get_state : virtual scenario_get_state_base
{
    virtual Object * operator() ();
    virtual ~default_get_state() {}
};


//    
struct scenario_get_state_id_base
{
    virtual String * operator() (Object*) = 0;
    virtual ~scenario_get_state_id_base() {}
};

struct default_get_state_id : virtual scenario_get_state_id_base
{
    virtual String * operator() (Object*);
    virtual ~default_get_state_id() {}
};

//   DFSM:  
struct scenario_save_model_state_base
{
    virtual Object * operator() () = 0;
    virtual ~scenario_save_model_state_base () {}
};

//   DFSM:  
struct scenario_restore_model_state_base
{
    virtual void operator() (Object*) = 0;
    virtual ~scenario_restore_model_state_base() {}
};

//   DFSM:   
struct scenario_is_stationary_state_base
{
    virtual bool operator() () = 0;
    virtual ~scenario_is_stationary_state_base() {}
};

//   DFSM:  
struct scenario_observe_state_base
{
    virtual void operator() () = 0;
    virtual ~scenario_observe_state_base() {}
};

//
//  ,    ++
//

//   
template<typename S>
struct scenario_init : virtual scenario_init_base
{
    typedef bool (S::*Init)(int argc, char ** argv);
protected:
    S& ref;
    Init init;
public:
    scenario_init(S& ref, Init init) : ref(ref), init(init) {}
    virtual bool operator() (int argc, char** argv)
    {
        return (ref.*init)(argc, argv);
    }
};

//      
template<typename S>
struct scenario_finish : virtual scenario_finish_base
{
    typedef void (S::*Finish)(void);
protected:
    S& ref;
    Finish finish;
public:
    scenario_finish(S&ref, Finish finish) : ref(ref), finish(finish) {}
    virtual void operator() ()
    {
        return (ref.*finish)();
    }
};

//    

template<typename S, typename T>
struct scenario_get_state : virtual scenario_get_state_base
{
    typedef T (S::*GetState)();
    S& ref;
    GetState method;
    scenario_get_state(S& scenario, GetState method) : ref(scenario), method(method) {}
    virtual Object * operator() ()
    {
        Object * res = cpptesk::atl::CppObject<T>::create((ref.*method)());
        return res;
    }
};

template<typename S, typename T>
struct scenario_get_state<S,T*> : virtual scenario_get_state_base
{
    typedef T* (S::*GetState)();
    S& ref;
    GetState method;
    scenario_get_state(S& scenario, GetState method) : ref(scenario), method(method) {}
    virtual Object * operator() ()
    {
        Object * res = cpptesk::atl::CppObject<T*>::create((ref.*method)());
        return res;
    }
};

template<typename S, typename T>
struct scenario_get_state<S, const T*> : virtual scenario_get_state_base
{
    typedef const T* (S::*GetState)();
    S& ref;
    GetState method;
    scenario_get_state(S& scenario, GetState method) : ref(scenario), method(method) {}
    virtual Object * operator() ()
    {
        Object * res = cpptesk::atl::CppObject<const T*>::create((ref.*method)());
        return res;
    }
};

template<typename S, typename T>
struct scenario_get_state<S,T&> : virtual scenario_get_state_base
{
    typedef T& (S::*GetState)();
    S& ref;
    GetState method;
    scenario_get_state(S& scenario, GetState method) : ref(scenario), method(method) {}
    virtual Object * operator() ()
    {
        Object * res = cpptesk::atl::CppObject<T*>::create(&((ref.*method)()));
        return res;
    }
};

template<typename S, typename T>
struct scenario_get_state<S,const T&> : virtual scenario_get_state_base
{
    typedef const T& (S::*GetState)();
    S& ref;
    GetState method;
    scenario_get_state(S& scenario, GetState method) : ref(scenario), method(method) {}
    virtual Object * operator() ()
    {
        Object * res = cpptesk::atl::CppObject<const T*>::create(&((ref.*method)()));
        return res;
    }
};

template<typename S>
struct scenario_get_state<S, Object*> : virtual scenario_get_state_base
{
    typedef Object* (S::*GetState)();
    S& ref;
    GetState method;
    scenario_get_state(S& scenario, GetState method) : ref(scenario), method(method) {}
    virtual Object * operator() ()
    {
        Object * res = (ref.*method)();
        return res;
    }
};

template<typename S>
struct scenario_get_state_id : virtual scenario_get_state_id_base
{
    typedef String* (S::*GetStateId)(Object*);
    S& ref;
    GetStateId method;
    scenario_get_state_id(S& scenario, GetStateId method) : ref(scenario), method(method) {}
    virtual String * operator() (Object* arg)
    {
        String * res = (ref.*method)(arg);
        return res;
    }
};

//    
template<typename S>
struct scenario_save_model_state : virtual scenario_save_model_state_base
{
    typedef Object * (S::*SaveModelState)();
    S& ref;
    SaveModelState method;
    scenario_save_model_state(S&ref, SaveModelState method) : ref(ref), method(method) {}
    Object * operator() ()
    {
        return (ref.*method)();
    }
};

template<typename S>
struct scenario_restore_model_state : virtual scenario_restore_model_state_base
{
    typedef void (S::*RestoreModelState)(Object *);
    S& ref;
    RestoreModelState method;
    scenario_restore_model_state(S&ref, RestoreModelState method) : ref(ref), method(method) {}
    virtual void operator() (Object * arg)
    {
        (ref.*method)(arg);
    }
};


template<typename S>
struct scenario_is_stationary_state : public scenario_is_stationary_state_base
{
    typedef bool (S::*IsStationaryState)();
    S& ref;
    IsStationaryState method;
    scenario_is_stationary_state(S&ref, IsStationaryState method) : ref(ref), method(method) {}
    virtual bool operator() ()
    {
        return (ref.*method)();
    }
};

template<typename S>
struct scenario_observe_state : virtual scenario_observe_state_base
{
    typedef void (S::*ObserveState)();
    S& ref;
    ObserveState method;

    scenario_observe_state(S& ref, ObserveState method) : ref(ref), method(method) {}
    virtual void operator() ()
    {
        (ref.*method)();
    }
};

}
}
