/** @file counter_scen.cpp
 * This file contains implementation of the scenario class.
 *
 * @author Mikhail Chupilko (chupilko@ispras.ru)
 */

#include <counter_scen.h>

#include <tracer/tracer.hpp>

namespace cpptesk {
namespace examples {
namespace counter {

bool CounterScenario::init(int argc, char ** argv)
{
    UNUSED_ARG(argc);
    UNUSED_ARG(argv);

    dut.initialize();
    return true;
}

void CounterScenario::finish()
{
    dut.finalize();
}

bool CounterScenario::scen_inc(Ctx& ctx)
{
    InputData msg;

    IBEGIN

    IACTION
    {
        dut.refresh_coverage();

        msg = InputData();

        dut.start(&Counter::inc, dut.iface1, msg, Counter::START_REQUEST_IFACE);
        dut.cycle();

        YIELD(dut.verdict());
    }
    IEND
}

std::string CounterScenario::get_state_counter()
{
    std::string state;
    std::stringstream out;

    out << "{counter:" << dut.get_value() << "}";
    state = out.str();
#ifdef DEBUG
    std::cout << "get_state_counter(): " << state << std::endl;
#endif
    return state;
}

CounterScenario::CounterScenario(void) : ScenarioBase()
{
}
CounterScenario::~CounterScenario(void) {}

ExampleScenario::ExampleScenario(void) : CounterScenario()
{
    setup("CounterScenario", &CounterScenario::init, &CounterScenario::finish, &CounterScenario::get_state_counter);

    ADD_SCENARIO_METHOD(CounterScenario::scen_inc);
}
ExampleScenario::~ExampleScenario(void) { };

}}}

